
#include "levels.h"
#include <fstream>

uint8_t const COLOUR_BITS = 0xC0;	//11000000
uint8_t const SINK_BIT = 0x20;		//00100000
uint8_t const DUDE_FACING_BITS = 0x18;	//00011000
uint8_t const DUDE_COLOUR_BIT = 0x04;	//00000100
uint8_t const DUDE_BIT = 0x02;		//00000010
uint8_t const ALL_BITS = 0xFE;		//11111110
uint8_t const NO_BITS = 0x00;		//00000000

uint8_t const IS_DARK = 0x00;		//00000000
uint8_t const IS_LIGHT = 0x80;		//10000000
uint8_t const IS_GREY = 0x40;		//01000000

uint8_t const IS_FEETDOWN = 0x00;	//00000000
uint8_t const IS_FEETRIGHT = 0x10;	//00010000
uint8_t const IS_FEETUP = 0x08;		//00001000
uint8_t const IS_FEETLEFT = 0x18;	//00011000

uint8_t const IS_DARK_DUDE = 0x00;	//00000000
uint8_t const IS_LIGHT_DUDE = 0x04;	//00000100

void Levels::load ()
{
	std::fstream dataFile;
	dataFile.open ("level_data", std::fstream::in | std::fstream::binary);
	
	for (int a = 0; a < MAX_LEVELS; ++a) {
		dataFile.read ((char*)&head_data [a], 1);
		for (int x = 0; x < GRID_SIZE; ++x)
			for (int y = 0; y < GRID_SIZE; ++y)
				if (dataFile.peek () != EOF)
					dataFile.read ((char*)&data [a][x][y], 1);
	}
	
	dataFile.close ();
}

void Levels::save ()
{
	std::fstream dataFile;
	dataFile.open ("level_data", std::fstream::out | std::fstream::binary | std::fstream::trunc);
	
	for (int a = 0; a < MAX_LEVELS; ++a) {
		dataFile.write ((char*)&head_data [a], 1);
		for (int x = 0; x < GRID_SIZE; ++x)
			for (int y = 0; y < GRID_SIZE; ++y)
				dataFile.write ((char*)&data [a][x][y], 1);
	}
	
	dataFile.close ();
}

int Levels::getWinNum ()
{
	return head_data [activeLevel];
}

int Levels::getColour (int x, int y)
{
	int colour = data [activeLevel][x - 1][y - 1] & COLOUR_BITS;
	
	if (colour == IS_LIGHT)
		return COLOUR_LIGHT;
	else if (colour == IS_DARK)
		return COLOUR_DARK;
	else
		return COLOUR_GREY;
}

bool Levels::getSink (int x, int y)
{
	return data [activeLevel][x - 1][y - 1] & SINK_BIT;
}

int Levels::getDudeFacing (int x, int y)
{
	int facing = data [activeLevel][x - 1][y - 1] & DUDE_FACING_BITS;
	
	if (facing == IS_FEETDOWN)
		return FEET_DOWN;
	else if (facing == IS_FEETRIGHT)
		return FEET_RIGHT;
	else if (facing == IS_FEETUP)
		return FEET_UP;
	else
		return FEET_LEFT;
}
		
int Levels::getDudeColour (int x, int y)
{
	int colour = data [activeLevel][x - 1][y - 1] & DUDE_COLOUR_BIT;
	
	if (colour == IS_LIGHT_DUDE)
		return COLOUR_LIGHT;
	else
		return COLOUR_DARK;
}

int Levels::getDudePresent (int x, int y)
{
	return data [activeLevel][x - 1][y - 1] & DUDE_BIT;
}

void Levels::setWinNum (int winNum)
{
	head_data [activeLevel] = winNum;
}

void Levels::setThingy (int x, int y, uint8_t goods, uint8_t mask)
{
	data [activeLevel][x - 1][y - 1] &= (mask ^ ALL_BITS);
	data [activeLevel][x - 1][y - 1] |= goods;
}

void Levels::setColour (int x, int y, int colour)
{
	if (colour == COLOUR_LIGHT)
		setThingy (x, y, IS_LIGHT, COLOUR_BITS);
	else if (colour == COLOUR_DARK)
		setThingy (x, y, IS_DARK, COLOUR_BITS);
	else
		setThingy (x, y, IS_GREY, COLOUR_BITS);
}

void Levels::setSink (int x, int y, bool sink)
{
	if (sink)
		setThingy (x, y, SINK_BIT, SINK_BIT);
	else
		setThingy (x, y, NO_BITS, SINK_BIT);
}

void Levels::setDudeFacing (int x, int y, int dudeFacing)
{
	if (dudeFacing == FEET_DOWN)
		setThingy (x, y, IS_FEETDOWN, DUDE_FACING_BITS);
	else if (dudeFacing == FEET_RIGHT)
		setThingy (x, y, IS_FEETRIGHT, DUDE_FACING_BITS);
	else if (dudeFacing == FEET_UP)
		setThingy (x, y, IS_FEETUP, DUDE_FACING_BITS);
	else
		setThingy (x, y, IS_FEETLEFT, DUDE_FACING_BITS);
}

void Levels::setDudeColour (int x, int y, int dudeColour)
{
	if (dudeColour == COLOUR_LIGHT)
		setThingy (x, y, IS_LIGHT_DUDE, DUDE_COLOUR_BIT);
	else
		setThingy (x, y, IS_DARK_DUDE, SINK_BIT);
}

void Levels::setDudePresent (int x, int y, bool dudePresent)
{
	if (dudePresent)
		setThingy (x, y, DUDE_BIT, DUDE_BIT);
	else
		setThingy (x, y, NO_BITS, DUDE_BIT);
}

