
#include "world.h"
#include "opengl_includes.h"
#include "stuff.h"
#include "levels.h"
#include <string>
#include <sstream>

static GLUquadricObj* qobj = 0;
static bool whichArbitraryColour = false;
static Levels* levels = 0;

void drawString (std::string s)
{
	for (int i = 0; s[i] != '\0'; ++i)
		glutBitmapCharacter (GLUT_BITMAP_TIMES_ROMAN_24, s[i]);
}

Dude::Dude (int in_colour, int in_facing, int in_xLoc, int in_yLoc)
{
	colour = in_colour;
	action = MOVING_RIGHT; // Will be changed immediately, but we still need a "prior action"
	facing = in_facing;
	xLoc = in_xLoc;
	yLoc = in_yLoc;
	position_step = 0;
	animation_step = 0;
}

void Dude::draw ()
{
	if (colour == COLOUR_LIGHT)
		glColor3f (0.0, 0.0, 0.0);
	else if (colour == COLOUR_DARK)
		glColor3f (1.0, 1.0, 1.0);
	else
		glColor3f (0.5, 0.5, 0.5);
		
	glPushMatrix ();
	
	glTranslatef (xLoc - 0.5, yLoc - 0.5, 0.0);
	
	float moved_distance = (float)position_step / (float)FRAMES_PER_CELL;
	if (action == MOVING_RIGHT)
		glTranslatef (moved_distance, 0.0, 0.0);
	else if (action == MOVING_LEFT)
		glTranslatef (-moved_distance, 0.0, 0.0);
	else if (action == MOVING_UP)
		glTranslatef (0.0, moved_distance, 0.0);
	else if (action == MOVING_DOWN)
		glTranslatef (0.0, -moved_distance, 0.0);
	
	if (facing == FEET_RIGHT)
		glRotatef (90.0, 0.0, 0.0, 1.0);
	else if (facing == FEET_UP)
		glRotatef (180.0, 0.0, 0.0, 1.0);
	else if (facing == FEET_LEFT)
		glRotatef (270.0, 0.0, 0.0, 1.0);
	
	float rotated_degrees = ((float)position_step / (float)FRAMES_PER_CELL) * 90.0;
	if (action == ROTATING_CLOCKWISE)
		glRotatef (-rotated_degrees, 0.0, 0.0, 1.0);
	else if (action == ROTATING_COUNTERCLOCKWISE)
		glRotatef (rotated_degrees, 0.0, 0.0, 1.0);
	
	glBegin (GL_LINES);
		glVertex2f (0.0, 0.4);
		glVertex2f (0.0, -0.2);
		
		glVertex2f (0.25, 0.0);
		glVertex2f (-0.25, 0.0);
		
		int slowstep = animation_step / 4;
		
		if (slowstep == 6) {
			glVertex2f (0.0, -0.2);
			glVertex2f (-0.3, -0.2);
			
			glVertex2f (0.0, -0.2);
			glVertex2f (0.3, -0.2);
		}
		if (slowstep == 5 || slowstep == 7) {
			glVertex2f (0.0, -0.2);
			glVertex2f (-0.28, -0.31);
			
			glVertex2f (0.0, -0.2);
			glVertex2f (0.28, -0.31);
		}
		if (slowstep == 4 || slowstep == 0) {
			glVertex2f (0.0, -0.2);
			glVertex2f (-0.21, -0.41);
			
			glVertex2f (0.0, -0.2);
			glVertex2f (0.21, -0.41);
		}
		if (slowstep == 3 || slowstep == 1) {
			glVertex2f (0.0, -0.2);
			glVertex2f (-0.11, -0.48);
			
			glVertex2f (0.0, -0.2);
			glVertex2f (0.11, -0.48);
		}
		if (slowstep == 2) {
			glVertex2f (0.0, -0.2);
			glVertex2f (0.0, -0.5);
		}
	glEnd ();
	gluQuadricDrawStyle (qobj, GLU_LINE);
	glTranslatef (0.0, 0.3, 0.0);
	gluDisk (qobj, 0.0, 0.1, 12, 1);
	glPopMatrix ();
}

void Dude::step ()
{
	position_step = (position_step + 1) % FRAMES_PER_CELL;
	animation_step = (animation_step + 1) % ANIMATION_FRAMES;
	
	if (position_step == 0)
		updateLocation ();
}

void Dude::updateLocation ()
{
	if (action == MOVING_RIGHT)
		xLoc++;
	else if (action == MOVING_LEFT)
		xLoc--;
	else if (action == MOVING_UP)
		yLoc++;
	else if (action == MOVING_DOWN)
		yLoc--;
	else if (action == ROTATING_CLOCKWISE) {
		if (facing == FEET_DOWN)
			facing = FEET_LEFT;
		else if (facing == FEET_LEFT)
			facing = FEET_UP;
		else if (facing == FEET_UP)
			facing = FEET_RIGHT;
		else // facing == FEET_RIGHT
			facing = FEET_DOWN;
	} else { // action == ROTATING_COUNTERCLOCKWISE
		if (facing == FEET_DOWN)
			facing = FEET_RIGHT;
		else if (facing == FEET_RIGHT)
			facing = FEET_UP;
		else if (facing == FEET_UP)
			facing = FEET_LEFT;
		else // facing == FEET_LEFT
			facing = FEET_DOWN;
	}
}

int Dude::disallowedColour ()
{
	if (colour == COLOUR_LIGHT)
		return COLOUR_DARK;
	else
		return COLOUR_LIGHT;
}

void Dude::getShifts (int& x_shift_front, int& y_shift_front, int& x_shift_floor, int& y_shift_floor, bool& wasMoving)
{
	int colourfix;

	if (colour == COLOUR_LIGHT)
		colourfix = 1;
	else
		colourfix = -1;

	x_shift_front = y_shift_front = x_shift_floor = y_shift_floor = 0;

	if (facing == FEET_DOWN) {
		x_shift_front = 1 * colourfix;
		y_shift_floor = -1;
	} else if (facing == FEET_UP) {
		x_shift_front = -1 * colourfix;
		y_shift_floor = 1;
	} else if (facing == FEET_RIGHT) {
		x_shift_floor = 1;
		y_shift_front = 1 * colourfix;
	} else { // facing == FEET_LEFT
		x_shift_floor = -1;
		y_shift_front = -1 * colourfix;
	}
	
	wasMoving = (action != ROTATING_CLOCKWISE && action != ROTATING_COUNTERCLOCKWISE);
}

void Dude::doThis (int act)
{
	if (act == DO_ROTATE_OPEN) {
		if (colour == COLOUR_LIGHT)
			action = ROTATING_CLOCKWISE;
		else
			action = ROTATING_COUNTERCLOCKWISE;
	} else if (act == DO_ROTATE_CLOSED) {
		if (colour == COLOUR_LIGHT)
			action = ROTATING_COUNTERCLOCKWISE;
		else
			action = ROTATING_CLOCKWISE;
	} else { // act == DO_MOVE
		if (colour == COLOUR_LIGHT) {
			if (facing == FEET_DOWN)
				action = MOVING_RIGHT;
			else if (facing == FEET_RIGHT)
				action = MOVING_UP;
			else if (facing == FEET_UP)
				action = MOVING_LEFT;
			else // facing == FEET_LEFT
				action = MOVING_DOWN;
		} else {
			if (facing == FEET_DOWN)
				action = MOVING_LEFT;
			else if (facing == FEET_LEFT)
				action = MOVING_UP;
			else if (facing == FEET_UP)
				action = MOVING_RIGHT;
			else // facing == FEET_RIGHT
				action = MOVING_DOWN;
		}
	}
}

World::World ()
{
	if (!qobj)
		qobj = gluNewQuadric ();
	if (!levels)
		levels = new Levels ();
		
	for (int w = 0; w <= GRID_SIZE + 1; ++w) {
		grid [w][0] = COLOUR_GREY;
		grid [w][GRID_SIZE + 1] = COLOUR_GREY;
		grid [0][w] = COLOUR_GREY;
		grid [GRID_SIZE + 1][w] = COLOUR_GREY;
	}
	
	toolDudes.push_back (new Dude (COLOUR_GREY, FEET_DOWN, 1, 0));
	toolDudes.push_back (new Dude (COLOUR_GREY, FEET_RIGHT, 2, 0));
	toolDudes.push_back (new Dude (COLOUR_GREY, FEET_UP, 3, 0));
	toolDudes.push_back (new Dude (COLOUR_GREY, FEET_LEFT, 4, 0));
	
	levels->load ();
	
	mode = MODE_MENU;
	initMode ();
}

void World::draw ()
{	
	for (int x = 1; x <= GRID_SIZE; x++)
		for (int y = 1; y <= GRID_SIZE; y++) {
			if (grid [x][y] != COLOUR_DARK) {
				grid [x][y] == COLOUR_LIGHT ?
					glColor3f (1.0, 1.0, 1.0) :
					glColor3f (0.5, 0.5, 0.5);
				glRecti (x, y, x - 1, y - 1);
			}
			if (sinks [x][y]) {
				glColor3f (0.0, 0.7, 0.5);
				gluQuadricDrawStyle (qobj, GLU_FILL);
				glPushMatrix ();
				glTranslatef (x - 0.5, y - 0.5, 0.0);
				gluDisk (qobj, 0.0, 0.25, 12, 1);
				glPopMatrix ();
			}
		}
	
	for (dude_set_t::iterator it = dudes.begin (); it != dudes.end (); ++it) {
		(*it)->draw ();
	}
	
	glColor3f (0.3, 0.3, 0.6);
	glRecti (0, 0, GRID_SIZE, -1);
	
	if (mode != MODE_MENU) {
		glColor3f (0.9, 0.1, 0.2);
		glBegin (GL_TRIANGLES);
			glVertex2f (10.1, -0.5);
			glVertex2f (10.5, -0.1);
			glVertex2f (10.5, -0.9);
		glEnd ();
		glRectf (10.5, -0.3, 10.9, -0.7);
	}
	
	if (mode == MODE_MENU) {
		glColor3f (0.9, 0.1, 0.2);
		for (int i = 0; i < 6; i++) {
			if (i != 2)
				glRectf (0.2, (11.0 - i) - 0.2, 0.8, (11.0 - i) - 0.8);
		}
		
		glRasterPos2f (1.2, 10.3);
		drawString ("Puzzle");
		glRasterPos2f (1.2, 9.3);
		drawString ("Freeplay");
		glRasterPos2f (1.2, 7.3);
		drawString ("Edit");
		glRasterPos2f (1.2, 6.3);
		drawString ("Reload");
		glRasterPos2f (1.2, 5.3);
		drawString ("Save");
	}
	
	if (mode == MODE_CHOOSELEVEL) {
		int levelNum = 1;
		for (int y = 9; y > 2; --y)
			for (int x = 3; x < 10; ++x) {
				levels->selectLevel (levelNum);
				levels->getWinNum () ?
					glColor3f (0.9, 0.2, 0.2) :
					glColor3f (0.4, 0.3, 0.3);
				glRasterPos2f (x - 0.7, y - 0.7);
				std::ostringstream s;
				s << levelNum;
				drawString (s.str ());
				++levelNum;
			}
	}
	
	if (mode == MODE_PUZZLE) {
		glColor3f (0.1, 0.8, 0.8);
		glRectf (0.4, -0.4, 0.6, -0.6);
		glRectf (2.4, -0.4, 2.6, -0.6);
		glRasterPos2f (0.9, -0.7);
		drawString ("Reset");
		glRasterPos2f (2.9, -0.7);
		drawString ("Skip");
		
		if (totalDudes == sunkDudes) {
			glRasterPos2f (2.9, 5.3);
			drawString ("Yay!  Click for next!");
		}
	}
	
	if (mode == MODE_FREEPLAY || mode == MODE_EDIT) {
		drawToolbar ();
	}
	
	if (mode == MODE_PUZZLE || mode == MODE_EDIT) {
		drawClicks ();
	}
}

void World::step ()
{
	if (mode == MODE_EDIT)
		return;
	
	if (mode == MODE_PUZZLE)
		if (delay > 0) {
			--delay;
			return;
		}
	
	dude_set_t::iterator it = dudes.begin ();
	while (it != dudes.end ()) {
		if ((*it)->ready ()) {
			int x, y;
			(*it)->getPosition (x, y);
			if (x == 0 | y == 0 | x == GRID_SIZE + 1 | y == GRID_SIZE + 1) {
				delete (*it);
				dudes.erase (it++);
			} else if (sinks [x][y]) {
				delete (*it);
				dudes.erase (it++);
				++sunkDudes;
			}else if (grid [x][y] == (*it)->disallowedColour ()) {
				delete (*it);
				dudes.erase (it++);
			}  else {
				int x_shift_front, y_shift_front, x_shift_floor, y_shift_floor;
				bool wasMoving;
				(*it)->getShifts (x_shift_front, y_shift_front, x_shift_floor, y_shift_floor, wasMoving);
				bool solidFront = grid [x + x_shift_front][y + y_shift_front] == (*it)->disallowedColour ();
				bool solidFloor = grid [x + x_shift_floor][y + y_shift_floor] == (*it)->disallowedColour ();
				bool solidDiagonal = grid [x + x_shift_front + x_shift_floor][y + y_shift_front + y_shift_floor] == (*it)->disallowedColour ();
				if (solidFront && solidFloor)
					(*it)->doThis (DO_ROTATE_CLOSED);
				else if (solidFloor || (solidDiagonal && !solidFront && !wasMoving))
					(*it)->doThis (DO_MOVE);
				else
					(*it)->doThis (DO_ROTATE_OPEN);
				++it;
			}
		} else { // not ready
			++it;
		}
	}
	
	for (dude_set_t::iterator it = dudes.begin (); it != dudes.end (); ++it) {
		(*it)->step ();
	}
}

void World::clicked (int x, int y)
{
	if (x == 11 && y == 0 && mode != MODE_MENU) {
		mode = MODE_MENU;
		initMode ();
		return;
	}
	if (mode == MODE_MENU) {
		if (x == 1) {
			if (y == 11) {
				mode = MODE_CHOOSELEVEL;
				forseen_mode = MODE_PUZZLE;
				initMode ();
			} else if (y == 10) {
				mode = MODE_FREEPLAY;
				initMode ();
			} else if (y == 8) {
				mode = MODE_CHOOSELEVEL;
				forseen_mode = MODE_EDIT;
				initMode ();
			} else if (y == 7) {
				levels->load ();
			} else if (y == 6) {
				levels->save ();
			}
		}
	} else if (mode == MODE_CHOOSELEVEL) {
		if (x >= 3 && x <=9 && y >= 3 && y <= 9) {
			levels->selectLevel ((x - 2) + ((9 - y) * 7));
			mode = forseen_mode;
			initMode ();
		}
	} else if (mode == MODE_PUZZLE) {
		if (x == 1 & y == 0) {
			initMode ();
		} else if (x == 3 & y == 0) {
			levels->incrementLevel ();
			initMode ();
		} else if (totalDudes == sunkDudes) {
			levels->incrementLevel ();
			initMode ();
		} else if (clicks > 0) {
			if (grid [x][y] != COLOUR_GREY) {
				flip (x, y);
				--clicks;
			}
		}
	} else if (mode == MODE_FREEPLAY) {
		if (y == 0) {
			if (x == TOOL_FEETDOWN ||
			x == TOOL_FEETRIGHT ||
			x == TOOL_FEETUP ||
			x == TOOL_FEETLEFT ||
			x == TOOL_SINK ||
			x == TOOL_LDFLIP ||
			x == TOOL_LDGFLIP) {
				activeTool = x;
			}
		} else {
			int dudeColour;
			
			if (activeTool == TOOL_FEETDOWN || activeTool == TOOL_FEETRIGHT || activeTool == TOOL_FEETUP || activeTool == TOOL_FEETLEFT) {
				if (grid [x][y] != COLOUR_GREY) 
					dudeColour = grid [x][y];
				else
					dudeColour = (whichArbitraryColour = !whichArbitraryColour) ? COLOUR_LIGHT : COLOUR_DARK;
			}
				
			if (activeTool == TOOL_FEETDOWN)
				dudes.insert (new Dude (dudeColour, FEET_DOWN, x, y));
			else if (activeTool == TOOL_FEETRIGHT)
				dudes.insert (new Dude (dudeColour, FEET_RIGHT, x, y));
			else if (activeTool == TOOL_FEETUP)
				dudes.insert (new Dude (dudeColour, FEET_UP, x, y));
			else if (activeTool == TOOL_FEETLEFT)
				dudes.insert (new Dude (dudeColour, FEET_LEFT, x, y));
			else if (activeTool == TOOL_SINK)
				sinks [x][y] = !sinks [x][y];
			else if (activeTool == TOOL_LDFLIP)
				flip (x, y);
			else if (activeTool == TOOL_LDGFLIP)
				flipWithGrey (x, y);
		}
	} else if (mode == MODE_EDIT) {
		if (y == 0) {
			if (x == TOOL_FEETDOWN ||
			x == TOOL_FEETRIGHT ||
			x == TOOL_FEETUP ||
			x == TOOL_FEETLEFT ||
			x == TOOL_SINK ||
			x == TOOL_LDFLIP ||
			x == TOOL_LDGFLIP) {
				activeTool = x;
			}
			if (x == CLICK_MINUS) {
				if (clicks > 0) {
					--clicks;
					levels->setWinNum (clicks);
				}
			}
			if (x == CLICK_PLUS) {
				if (clicks < 99) {
					++clicks;
					levels->setWinNum (clicks);
				}
			}
		} else {
			int dudeColour;
			Dude* found_dude;
			
			if (activeTool == TOOL_FEETDOWN || activeTool == TOOL_FEETRIGHT || activeTool == TOOL_FEETUP || activeTool == TOOL_FEETLEFT) {
				
				if (found_dude = findDude (x, y)) {
					dudes.erase (found_dude);
					levels->setDudePresent (x, y, false);
				} else {
					if (grid [x][y] != COLOUR_GREY) 
						dudeColour = grid [x][y];
					else
						dudeColour = (whichArbitraryColour = !whichArbitraryColour) ? COLOUR_LIGHT : COLOUR_DARK;
					
					levels->setDudePresent (x, y, true);
					levels->setDudeColour (x, y, dudeColour);
					if (activeTool == TOOL_FEETDOWN) {
						dudes.insert (new Dude (dudeColour, FEET_DOWN, x, y));
						levels->setDudeFacing (x, y, FEET_DOWN);
					} else if (activeTool == TOOL_FEETRIGHT) {
						dudes.insert (new Dude (dudeColour, FEET_RIGHT, x, y));
						levels->setDudeFacing (x, y, FEET_RIGHT);
					} else if (activeTool == TOOL_FEETUP) {
						dudes.insert (new Dude (dudeColour, FEET_UP, x, y));
						levels->setDudeFacing (x, y, FEET_UP);
					} else { // activeTool == TOOL_FEETLEFT
						dudes.insert (new Dude (dudeColour, FEET_LEFT, x, y));
						levels->setDudeFacing (x, y, FEET_LEFT);
					}
				}
			}
			
			if (activeTool == TOOL_SINK) {
				sinks [x][y] = !sinks [x][y];
				levels->setSink (x, y, sinks [x][y]);
			} else if (activeTool == TOOL_LDFLIP) {
				flip (x, y);
				levels->setColour (x, y, grid [x][y]);
			} else if (activeTool == TOOL_LDGFLIP) {
				flipWithGrey (x, y);
				levels->setColour (x, y, grid [x][y]);
			}
		}
	}
}

void World::flip (int x, int y)
{
	if (x < 1 || y < 1 || x > GRID_SIZE || y > GRID_SIZE)
		return;
		
	if (grid [x][y] == COLOUR_LIGHT)
		grid [x][y] = COLOUR_DARK;
	else if (grid [x][y] == COLOUR_DARK)
		grid [x][y] = COLOUR_LIGHT;
}

void World::flipWithGrey (int x, int y)
{
	if (x < 1 || y < 1 || x > GRID_SIZE || y > GRID_SIZE)
		return;
		
	if (grid [x][y] != COLOUR_GREY)
		grid [x][y] = COLOUR_GREY;
	else
		grid [x][y] = ((whichArbitraryColour = !whichArbitraryColour) ? COLOUR_LIGHT : COLOUR_DARK);
}

void World::clearDudes ()
{
	for (dude_set_t::iterator it = dudes.begin (); it != dudes.end (); ++it) {
		delete (*it);
	}
	dudes.clear ();
}

void World::initMode ()
{
	if (mode == MODE_MENU) {
		clearDudes ();
		for (int x = 1; x <= GRID_SIZE; ++x)
			for (int y = 1; y <= GRID_SIZE; ++y) {
				grid [x][y] = (x < 5 || x > 7 || y < 5 || y > 7) ? COLOUR_LIGHT : COLOUR_DARK;
				sinks [x][y] = false;
			}
		dudes.insert (new Dude (COLOUR_LIGHT, FEET_DOWN, 6, 8));
		dudes.insert (new Dude (COLOUR_DARK, FEET_UP, 6, 7));
	}
	if (mode == MODE_CHOOSELEVEL) {
		// Nothing to do?
	}
	if (mode == MODE_PUZZLE || mode == MODE_EDIT) {
		clearDudes ();
		
		for (int x = 1; x <= GRID_SIZE; ++x)
			for (int y = 1; y <= GRID_SIZE; ++y) {
				grid [x][y] = levels->getColour (x, y);
				sinks [x][y] = levels->getSink (x, y);
				if (levels->getDudePresent (x, y))
					dudes.insert (new Dude (levels->getDudeColour (x, y), levels->getDudeFacing (x, y), x, y));
			}
			
		clicks = levels->getWinNum ();
	}
	if (mode == MODE_PUZZLE) {
		delay = 60;
		sunkDudes = 0;
		totalDudes = dudes.size ();
	}
	if (mode == MODE_EDIT) {
		activeTool = TOOL_LDFLIP;
	}
	if (mode == MODE_FREEPLAY) {
		clearDudes ();
		
		for (int x = 1; x <= GRID_SIZE; ++x)
			for (int y = 1; y <= GRID_SIZE; ++y) {
				grid [x][y] = (x < 6) ? COLOUR_LIGHT : COLOUR_DARK;
				sinks [x][y] = false;
			}
		for (int y = 11; y > 6; --y)
			grid [6][y] = COLOUR_LIGHT;
		
		grid [6][6] = COLOUR_GREY;
		
		activeTool = TOOL_LDFLIP;
	}
}

void World::drawToolbar ()
{
	for (int i = 0; i < 4; ++i)
		toolDudes [i]->draw ();
	
	glColor3f (0.0, 0.7, 0.5);
	gluQuadricDrawStyle (qobj, GLU_FILL);
	glPushMatrix ();
	glTranslatef (4.5, -0.5, 0.0);
	gluDisk (qobj, 0.0, 0.25, 12, 1);
	glPopMatrix ();
	
	glColor3f (0.0, 0.0, 0.0);
	glBegin (GL_QUADS);
		glVertex2f (5.2, -0.2);
		glVertex2f (5.6, -0.2);
		glVertex2f (5.4, -0.8);
		glVertex2f (5.2, -0.8);
	glEnd ();
	glColor3f (1.0, 1.0, 1.0);
	glBegin (GL_QUADS);
		glVertex2f (5.6, -0.2);
		glVertex2f (5.8, -0.2);
		glVertex2f (5.8, -0.8);
		glVertex2f (5.4, -0.8);
	glEnd ();
	
	glColor3f (0.0, 0.0, 0.0);
	glBegin (GL_QUADS);
		glVertex2f (6.2, -0.2);
		glVertex2f (6.5, -0.2);
		glVertex2f (6.3, -0.8);
		glVertex2f (6.2, -0.8);
	glEnd ();
	glColor3f (0.5, 0.5, 0.5);
	glBegin (GL_QUADS);
		glVertex2f (6.5, -0.2);
		glVertex2f (6.7, -0.2);
		glVertex2f (6.5, -0.8);
		glVertex2f (6.3, -0.8);
	glEnd ();
	glColor3f (1.0, 1.0, 1.0);
	glBegin (GL_QUADS);
		glVertex2f (6.7, -0.2);
		glVertex2f (6.8, -0.2);
		glVertex2f (6.8, -0.8);
		glVertex2f (6.5, -0.8);
	glEnd ();
	
	glColor3f (0.2, 0.9, 0.1);
	glRectf (activeTool - 0.05, -0.05, activeTool - 0.15, -0.95);
	glRectf (activeTool - 0.85, -0.05, activeTool - 0.95, -0.95);
	glRectf (activeTool - 0.05, -0.05, activeTool - 0.95, -0.15);
	glRectf (activeTool - 0.05, -0.85, activeTool - 0.95, -0.95);
}

void World::drawClicks ()
{
	glColor3f (0.8, 0.0, 0.7);
	glRasterPos2f (8.2, -0.7);
	std::ostringstream s;
	s << clicks;
	drawString (s.str ());
	
	if (mode == MODE_EDIT) {
		glRectf (9.45, -0.35, 9.55, -0.65);
		glRectf (7.35, -0.45, 7.65, -0.55);
		glRectf (9.35, -0.45, 9.65, -0.55);
	}
}

Dude* World::findDude (int x, int y)
{
	for (dude_set_t::iterator it = dudes.begin (); it != dudes.end (); ++it) {
		int dudeX, dudeY;
		(*it)->getPosition (dudeX, dudeY);
		if (x == dudeX && y == dudeY)
			return (*it);
	}
	return 0;
}
